#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include "relais.h"

const unsigned int RELAIS_COUNTER = 5;
Relais * relais[] = {
  new Relais("R1", D12),
  new Relais("R2", D11),
  new Relais("R3", D10),
  new Relais("R4", D9),
  new Relais("R5", D8),  
  };

const unsigned long MAX_ACTIVE_RELAY_TIME = 5000; //5sek.

const unsigned int MAX_TABLE_COLUMNS = 3;

const char* ssid = ""; //SSID aus dem Router
const char* password = ""; //Passwort für den Zugang zum WLAN
 
ESP8266WebServer server(80); 

void setup() {
  Serial.begin(115200); //Baudrate für die Serielle Geschwindigkeit.
  delay(10); 
   
  Serial.print("Aufbau der Verbindung zu: "); //Ausgabe der SSID auf der Seriellen Schnittstelle.
  Serial.println(ssid);
  
  WiFi.begin(ssid, password); //Initialisieren der Wifi Verbindung.
 
  while (WiFi.status() != WL_CONNECTED) { //Warten bis die Verbindung aufgebaut wurde.
    delay(500);
    //Einen Punkt auf der Seriellen Schnittstelle ausgeben so das der Benutzer erkennt dass, das Sketch noch läuft.
    Serial.print("."); 
  }
  //Bei erfolgreicher Verbindung wird der folgende Text ausgeben.
  Serial.println("");
  Serial.print("Mit ");
  Serial.print(ssid);
  Serial.print(" erfolgreich verbunden!");
   
  server.begin(); // Starten des Servers.
  Serial.println("");
  Serial.println("Server gestartet"); //Ausgabe auf der Seriellen Schnittstelle das der Server gestartet wurde.
  Serial.println("");
  
  // Ausgabe der IP Adresse 
  Serial.print("Adresse : http://");
  Serial.print(WiFi.localIP());
  Serial.println("/");

  server.on("/", callHome);
  server.on("/set", callSet);

  for(int i=0;i<RELAIS_COUNTER;i++){
   pinMode(relais[i] -> getPin(),OUTPUT);
  }
}

void callHome(){
  String content = "";
  content += "<!DOCTYPE HTML>";
  content += "<html>"; 
  content += "<head>"; 
  content += "<style>"; 
  content += ".wrapper {max-width:90%;margin:0 auto;text-align:center;}"; 
  content += ".btn {margin:15px;}";   
  content += "</style>"; 
  content += "</head>"; 
  content += "<body>"; 
  content += "<div class='wrapper'>"; 
  content += "<h2>Pyro - Entferno</h2>"; 
  content += "<h2>v3.0</h2>"; 
  content += "<br/><br/>"; 
  content += "<center>"; 
  for(int i=0;i<RELAIS_COUNTER;i++){
    Relais * currentRelais = relais[i];
    content += "<input type='button' class= 'btn' value='"+currentRelais -> getBezeichnung()+"' onClick='sendRequest("+i+")'/>"; 
    int value = (i+1) % MAX_TABLE_COLUMNS;
    if(value == 0){
      content += "<div style='clear:both;'></div>"; 
    }
  }
  String ipAddress = WiFi.localIP().toString();
  content += "</center>"; 
  content += "</div>"; 
  content += "<script>"; 
  content += "const url='http://"+ipAddress+"/set?';"; 
  content += "function sendRequest(button){"; 
  content += "  const Http = new XMLHttpRequest();";
  content += "  reqUrl=url+'btn='+button;";
  content += "  console.log(reqUrl);";
  content += "  Http.open('GET', reqUrl);";
  content += "  Http.send();";
  content += "}"; 
  content += "</script>"; 
  content += "</body>"; 
  content += "</html>"; 
   
   sendResult(content);
}

void callSet(){
  int buttonIndex = -1;
 
  for (int i = 0; i < server.args(); i++) {
      String parameterName = server.argName(i);
      String parameterValue = server.arg(i);
 
      if(parameterName == "btn"){
        buttonIndex = parameterValue.toInt();
      }
  }

  if(buttonIndex > -1){
    Relais * currentRelais = relais[buttonIndex];
    currentRelais -> setStartTime(millis());
    digitalWrite(currentRelais -> getPin(),HIGH);
    sendResult("{\"msg\": \"Das Relais mit der Bezeichnung "+currentRelais -> getBezeichnung()+" ist aktiv!\"}");
  }  
  
}

void sendResult(String content){
  server.send(200, "text/html", content);  
}
 
void loop() {
  server.handleClient();

   for(int i=0;i<RELAIS_COUNTER;i++){
      Relais * currentRelais = relais[i];
      long startTime = currentRelais -> getStartTime();
      if(startTime > -1){
        long currentTime = millis();
        if((currentTime - MAX_ACTIVE_RELAY_TIME)> startTime){
          digitalWrite(currentRelais -> getPin(),LOW);
          currentRelais -> setStartTime(-1);
        }
      }
   } 
}
